import tkinter as tk
from tkinter import messagebox
import smartcomputer

class TicTacToe:
    def __init__(self, master):
        self.master = master
        self.master.title("井字棋")
        self.pieces = 'XO'
        self.win_steps = smartcomputer.win_steps
        self.board = [tk.StringVar(value='') for i in range(9)]
        self.steps, self.buttons = [], []
        for i, item in enumerate(self.board):
            button = tk.Button(self.master, textvariable=self.board[i], font=("Arial", 20), width=5, height=2,
                               command=lambda x=i: self.human_play(x))
            button.grid(row=i // 3, column=i % 3)
            self.buttons.append(button)
        self.init()

    def init(self):
        [item.set('') for item in self.board if len(self.steps) > 0]
        self.steps.clear()
        self.buttons.clear()
        self.human_flag, self.current_player = False, 0
        if not self.human_flag: self.computer_play()

    def human_play(self, x):
        if not self.human_flag or x in self.steps: return
        self.play(x)

    def computer_play(self):
        #x = smartcomputer.random_play(self.steps)
        x = smartcomputer.smart_play(self.steps,self.current_player)
        self.play(x)


    def play(self, x):
        self.steps.append(x)
        self.board[x].set(self.pieces[self.current_player])
        r = self.check()
        if r:
            self.show_result(r)
        else:
            self.human_flag = not self.human_flag
            self.current_player = 1 if self.current_player == 0 else 0
            if not self.human_flag: self.computer_play()

    def show_result(self, r):
        message = ['和棋', '双方已走成和棋！'] if r == 1 else ['胜利',f'恭喜 {self.pieces[self.current_player]} 获得胜利！']
        messagebox.showinfo(message[0], message[1])
        self.init()

    def check(self):
        if len(self.steps) < 3: return 0
        data = {step for i, step in enumerate(self.steps) if i % 2 == self.current_player}
        r = 2 if any([item.issubset(data) for item in self.win_steps]) else 0
        r = 1 if r==0 and len(self.steps)>8 else r
        return r

if __name__ == "__main__":
    root = tk.Tk()
    game = TicTacToe(root)
    root.mainloop()